@@ -14,7 +14,7 @@
 //       * invlogit_AK
 //       * log_AK, log0_AK
 //       * cumsum
-//       * sum
+//       * sum (OVERLOADED)
 //       * prod
 //       * maxArray (OVERLOADED)
 //       * fillArray (OVERLOADED)
@@ -50,6 +50,7 @@ const double _ZERO_DOUBLE = 0;
 const double _NORM_ZERO = 1e-16;                                                  // qnorm(1 - 1e-16) is still non infty in R
 const double _ZERO  = 1e-50;                                                      // used by log_AK
 const double _ZERO0 = 1e-305;                                                     // used by log0_AK
+const double _LOG_ZERO0 = log(_ZERO0);                                            // used by log0_AK_bound
 const double _invFLT_MAX = 1e-50;
 const double _SCALE_ZERO = 1e-20;
 
@@ -73,24 +74,19 @@ ident_AK(const double& x){
   return(x);
 };
 
-/*** exp(x) ***/
-inline double
-exp_AK(const double& x){
-  return (x < _EMIN ? 0.0 : (x > _EMAX ? R_PosInf : exp(x)));
-};
-
-/*** log(x) ***/
-inline double
-log_AK(const double& x){
-  return(x < _ZERO ? R_NegInf : log(x));
-}
-
 /*** exp(x)/(1 + exp(x)) ***/
 inline double
 invlogit_AK(const double& x){
   const double exp_x=exp(x);
   return (x < _EMIN ? 0.0 : (x > _EMAX ? 1.0 : exp_x/(1 + exp_x)));
 };
+
+/*** exp(x) ***/
+inline double
+exp_AK(const double& x){
+  return (x < _EMIN ? 0.0 : (x > _EMAX ? R_PosInf : exp(x)));
+};
+
  
 /*** exp(x), returning something > 0 even for very negative values ***/
 inline double
@@ -98,13 +94,38 @@ exp0_AK(const double& x){
   return (x < _EMIN0 ? 0.0 : (x > _EMAX ? R_PosInf : exp(x)));
 };
 
+/*** log(x) ***/
+inline double
+log_AK(const double& x){
+  return(x < _ZERO ? R_NegInf : log(x));
+}
+
 /*** log(x) ***/
 inline double
 log0_AK(const double& x){
   return(x < _ZERO0 ? R_NegInf : log(x));
 }
 
-/*** sum of elements of an array ***/
+/*** log(x) ***/
+inline double
+log0_AK_bound(const double& x){
+  return(x < _ZERO0 ? _LOG_ZERO0 : log(x));
+}
+
+/*** sum of elements of an array (overloaded function) ***/
+inline double
+sum(const double* x,  const int& nx){
+  static double i;
+  const double *xP = x;
+  double VALUE = *xP;
+  for (i = 1; i < nx; i++){
+    xP++;
+    VALUE += *xP;  
+  }
+
+  return(VALUE);
+}
+
 inline int
 sum(const int* x,  const int& nx){
   static int i;
